close all
clear all
clc
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% MSeq generator
% 6 bit seed - 63 bits length
%
% List of Primitive polynomials, as listed in
% https://www.embeddedrelated.com/showarticle/1193.php
% Linear Feedback Shift Registers for the Uninitiated _  Sachs -2018 -2024
%
% Set of Primitive Polynomials:
% Primitive Poly ==  67  0103  43h  1000011     x6+ x+ 1
% Primitive Poly ==  91  0133  5bh  1011011     x6+x4+x3+ x+ 1
% Primitive Poly ==  97  0141  61h  1100001     x6+x5+ 1
% Primitive Poly == 103  0147  67h  1100111     x6+x5+x2+ x+ 1
% Primitive Poly == 109  0155  6dh  1101101     x6+x5+x3+x2+ 1
% Primitive Poly == 115  0163  73h  1110011     x6+x5+x4+ x+ 1


SREGLEN= 6;                     % Shift Register Length is 6 Bits -> MSeq is 63 bits long
qMSeqLen=2^(SREGLEN) -1;        % MSeq Length== 63

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
tM63Seq067=zeros(qMSeqLen, qMSeqLen);     % 63 kets with 63 values each
tM63Seq091=zeros(qMSeqLen, qMSeqLen);     
tM63Seq097=zeros(qMSeqLen, qMSeqLen);     
tM63Seq103=zeros(qMSeqLen, qMSeqLen);     
tM63Seq109=zeros(qMSeqLen, qMSeqLen);     
tM63Seq115=zeros(qMSeqLen, qMSeqLen);     

for pp= [67, 91, 97, 103, 109, 115]   
    qPrimPoly= pp;          % Set the primitive polynomial
    % Set up the Prim Poly Weights
    bPrimPW= de2bi(floor(qPrimPoly/2),SREGLEN );    %x6+ x
    
    % Shift register bot ini (bots are bits coded as -1 and +1)
    for ss= 0:(qMSeqLen-1)    %  Must be 0<= ss <= (qMSeqLen-1)
        bSh= -1 +2*flip(de2bi(ss, SREGLEN));
        
        %Generate the MSeq
        kMSeq= zeros(qMSeqLen,1);
        for nn= 1: qMSeqLen
            x0= 1;      % Init to neutral
            if bPrimPW(6); x0= -x0 *bSh(6); end
            if bPrimPW(5); x0= -x0 *bSh(5); end
            if bPrimPW(4); x0= -x0 *bSh(4); end
            if bPrimPW(3); x0= -x0 *bSh(3); end
            if bPrimPW(2); x0= -x0 *bSh(2); end
            if bPrimPW(1); x0= -x0 *bSh(1); end
            
            kMSeq(nn)= x0;
            bSh=[ x0, bSh(1:5)];  %Feedback Shift
        end
        
        if(pp==  67); tM63Seq067(:,ss+1)= kMSeq ; end
        if(pp==  91); tM63Seq091(:,ss+1)= kMSeq ; end
        if(pp==  97); tM63Seq097(:,ss+1)= kMSeq ; end
        if(pp== 103); tM63Seq103(:,ss+1)= kMSeq ; end
        if(pp== 109); tM63Seq109(:,ss+1)= kMSeq ; end
        if(pp== 115); tM63Seq115(:,ss+1)= kMSeq ; end
        stem(cconv(kMSeq, flip(kMSeq), qMSeqLen))
        xlabel(ss); title(qPrimPoly)
        pause(0.25);
    end
end

save M63SeqsAll tM63Seq067 tM63Seq091 tM63Seq097 tM63Seq103 tM63Seq109 tM63Seq115


