close all
clear all
clc
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% MSeq generator
% 6 bit seed - 63 bits length
%
% List of Primitive polynomials, as listed in
% https://www.embeddedrelated.com/showarticle/1193.php
% Linear Feedback Shift Registers for the Uninitiated _  Sachs -2018 -2024
%
% Set of Primitive Polynomials:
% Primitive Poly ==  67  0103  43h  1000011     x6+ x+ 1
% Primitive Poly ==  91  0133  5bh  1011011     x6+x4+x3+ x+ 1
% Primitive Poly ==  97  0141  61h  1100001     x6+x5+ 1
% Primitive Poly == 103  0147  67h  1100111     x6+x5+x2+ x+ 1
% Primitive Poly == 109  0155  6dh  1101101     x6+x5+x3+x2+ 1
% Primitive Poly == 115  0163  73h  1110011     x6+x5+x4+ x+ 1


SREGLEN= 6;                     % Shift Register Length is 6 Bits -> MSeq is 63 bits long
qMSeqLen=2^(SREGLEN) -1;        % MSeq Length== 63

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

for pp= 1:2:127
    close
    qPrimPoly= pp;          % Set the primitive polynomial
    % Set up the Prim Poly Weights
    bPrimPW= de2bi(floor(qPrimPoly/2),SREGLEN );    %x6+ x
    
    % Shift register bot ini (bots are bits coded as -1 and +1)
    for ss= 0:(qMSeqLen-1)    %  Must be 0<= ss <= (qMSeqLen-1)
        bSh= -1 +2*flip(de2bi(ss, SREGLEN));
        
        %Generate the MSeq
        kMSeq= zeros(qMSeqLen,1);
        for nn= 1: qMSeqLen
            x0= 1;      % Init to neutral
            if bPrimPW(6); x0= -x0 *bSh(6); end
            if bPrimPW(5); x0= -x0 *bSh(5); end
            if bPrimPW(4); x0= -x0 *bSh(4); end
            if bPrimPW(3); x0= -x0 *bSh(3); end
            if bPrimPW(2); x0= -x0 *bSh(2); end
            if bPrimPW(1); x0= -x0 *bSh(1); end
            
            kMSeq(nn)= x0;
            bSh=[ x0, bSh(1:5)];  %Feedback Shift
        end
        
        kACorr= cconv(kMSeq, flip(kMSeq), qMSeqLen);
        kACorrL= kACorr(1:62);
        
        if var(kACorrL)< (512*eps)
            if (1+ mean(kACorrL)) < (512*eps)
                stem(kACorr)
                xlabel(ss); title(qPrimPoly)
                if ss== 0; disp(qPrimPoly); end  % Display Prim Poly found
                pause(0.01);
            end
        end
    end
end



