clear
qStartTime= datetime;
close all
clc
%
DATAFILE= 'OS23Pos.mat';
%Logic constants
FALSE= (0==1);
TRUE= (1==1);
%
%Sigma parameters
SIGMAINI= 0.2;
SIGMASTEP= 0.995;
MINSEARCHSIGMA= 100*eps;
%
% Max iteration count
NNMAX= 10^6;
%
%
%                s + z1
% H(s)= qGain -------------------- exp(-sT0)
%             s2 + qBs + qC
%
%

load(DATAFILE)
qTs= 1/16384;           % Sampling period
kYData= OS23Pos;        % Data points
%
qNN= numel(kYData);
kK= (1:qNN)';
kTt= qTs*(kK-1);        % Time axis starts at zero
qT0Max= kTt(end);



%-----------------------------------------

%We would like to fit, in the least error energy sense, the function
% kYEst= qGain*step(t-T0)
% to the data.
% Two unknow parameters:  qGain, qT0

% Generate the tentative best Gain and T0 values:
qBestT0= qT0Max/2;
qBestGain= (max(max(kYData)) + min(min(kYData)) )/2;

% Compute the first best estimate
kBestYEst = qBestGain * ((1+ sign(kTt-qBestT0))/2);

% Compute the first best error energy
kBestErr= kBestYEst - kYData;  % Y== YEst-Err
qBestErrPower= kBestErr'*kBestErr;

% Plot, just to show something
figure(1)
plot(kTt, kYData, kTt, kBestYEst);
pause(0.1)

% Search loop
bolSearchGainT0= TRUE;
nnGainT0= 0;
qSearchSigma= SIGMAINI;
while bolSearchGainT0 ~= FALSE
    nnGainT0= nnGainT0+1;
    
    % Generate new Gain and T0
    kSeed= randn(4,1);
    qNewGain= qBestGain*(1 + qSearchSigma*kSeed(1)) + qSearchSigma*kSeed(2);
    qNewT0= abs(qBestT0*(1 + qSearchSigma*kSeed(3)) + qSearchSigma*kSeed(4));
    while qNewT0 > qT0Max; qNewT0= qNewT0- qT0Max; end  % Boundaries
    
    % Compute new YEst
    kNewYEst = qNewGain * ((1+ sign(kTt-qNewT0))/2);
    
    % Compute the new error energy
    kNewErr= kNewYEst - kYData;  % Y== YEst-Err
    qNewErrPower= kNewErr'*kNewErr;
    
    % Look for improvement
    if qNewErrPower < qBestErrPower
        
        qBestErrPower= qNewErrPower;    % New best values
        kBestYEst= kNewYEst;        
        kBestErr= kNewErr;
        qBestGain= qNewGain;
        qBestT0= qNewT0;
        
        nnGainT0Sol= nnGainT0;      % New metadata
        qSolTime= datetime;
        
        %Decrease SearchSigma
        qSearchSigma= qSearchSigma*SIGMASTEP;
        
        % progress plot
        figure(1)
        plot(kTt, kYData, kTt, kBestYEst);
        xlabel(sprintf('nn= %d, Log err power= %8.4f ',nnGainT0, log10(qBestErrPower)))
        title(DATAFILE);
        pause(0.1)
        
    end
    
    
    
    % Stop search if to many iterations
    if nnGainT0 > NNMAX; bolSearchGainT0= FALSE; end
    
    % Stop search if SearchSigma is too small
    if qSearchSigma < MINSEARCHSIGMA; bolSearchGainT0= FALSE; end
end


figure(1)
plot(kTt, kYData, kTt, kBestYEst);
xlabel(sprintf('Sol: nn= %d, Log err power= %8.4f ',nnGainT0Sol, log10(qBestErrPower)))
title(DATAFILE);
pause(0.1)

%------------------------------------------

fprintf('qGainBest= %7.4f\n',qBestGain);
fprintf('qT0Best= %7.4f\n',qBestT0);

disp(' ')
qEndTime= datetime;
qRunElapsedTime= between(qStartTime, qEndTime);
qSolElasedTime= between(qStartTime, qSolTime);
disp('Solution found after: ');
disp(qSolElasedTime);
disp(' ')
disp('Run duration: ');
disp(qRunElapsedTime);


