clear
qStartTime= datetime;
close all
clc
%
%Logic constants
FALSE= (0==1);
TRUE= (1==1);
%
%Sigma parameters
SIGMAINI= .3;
SIGMASTEP= 0.995;
MINSEARCHSIGMA= 100*eps;
%
% Max iteration count
NNMAX= 10^8;
MAXNOPROGRESS= fix(NNMAX/100);

% Min power reduction
MINPOWREDUX= 10^(-3);
% Set of three redux flags
bolReduxFlag0= FALSE;
bolReduxFlag1= FALSE;
bolReduxFlag2= FALSE;

% Exit reasons
NOPOWREDUX= 1;
TOOMANYITERS= 2;
TOOSMALLSIGMA= 4;
NOPROGRESS= 8;


% Data management
DATAFILE= 'OS23Neg.mat';
load(DATAFILE)
kYData= OS23Neg;        % Data points
qTs= 1/16384;           % Sampling period
%
qNN= numel(kYData);
kK= (1:qNN)';
kTt= qTs*(kK-1);        % Time axis starts at zero
qT0Max= kTt(end);



%----------------------------------------
%We would like to fit, in the least error power sense, the function
% kYEst= qGain*step(t-T0).*(R1*exp(P1*(t-T0)) + conj(R1)*exp(conj(P1)*(t-T0)) + R3; % 1== exp(0*(t-qBestT0))
% to the data.
% Five unknow parameters:  qGain, qT0, R1, P1, R3

% Generate the tentative best Gain and T0 values:
qT0Ini= qT0Max/2;                       qBestT0= qT0Ini;
qGainIni= (max(max(kYData)) + min(min(kYData)) )/2;
qBestGain= qGainIni;
% Generate the tentative best R1, P1 and R3 values:
qR1Ini= qBestT0 + 1i*qBestT0;           qBestR1= qR1Ini;
qP1Ini= 1/(-qBestT0 + 1i*qBestT0);      qBestP1= qP1Ini;
qR3Ini= qBestT0;                        qBestR3= qR3Ini;


% Compute the first best estimate
kBestYEst = qBestGain * ((1+ sign(kTt-qBestT0))/2).*...
    (qBestR1*exp(qBestP1*(kTt-qBestT0)) + conj(qBestR1)*exp(conj(qBestP1)*(kTt-qBestT0)) + qBestR3); % 1== exp(0*(kTt-qBestT0)));

% Compute the first best error power
kBestErr= kBestYEst - kYData;  % Y== YEst-Err
qBestErrPower= kBestErr'*kBestErr;

% Initial Plot 
figure(1)
plot(kTt, kYData, kTt, kBestYEst);
pause(0.1)

% Search loop
bolSearch= TRUE;
nnSearch= 0;
nnNoProgress= 0;
qSearchSigma= SIGMAINI;
while bolSearch ~= FALSE
    nnSearch= nnSearch+1;
    nnNoProgress= nnNoProgress+1;
    
    % Generate new search values
    kSeed= randn(12,1);
    qNewGain= qBestGain*(1 + qSearchSigma*kSeed(1)) + qSearchSigma*kSeed(2);
    qNewT0= abs(qBestT0*(1 + qSearchSigma*kSeed(3)) + qSearchSigma*kSeed(4));
    while qNewT0 > qT0Max; qNewT0= qNewT0- qT0Max; end  % Boundaries
    
    qNewR1= qBestR1*(1 + qSearchSigma*kSeed(5)) + qSearchSigma*kSeed(6) + 1i* qSearchSigma*kSeed(7);
    qNewP1= qBestP1*(1 + qSearchSigma*kSeed(8)) + qSearchSigma*kSeed(9) + 1i* qSearchSigma*kSeed(10);
    if real(qNewP1) > 0; qNewP1= -real(qNewP1) +1i*imag(qNewP1); end %  % Must be negative for Stability
    qNewR3= qBestR3*(1 + qSearchSigma*kSeed(11)) + qSearchSigma*kSeed(12);
    
    % Compute new YEst
    kNewYEst = qNewGain * ((1+ sign(kTt-qNewT0))/2).*...
        (qNewR1*exp(qNewP1*(kTt-qNewT0)) + conj(qNewR1)*exp(conj(qNewP1)*(kTt-qNewT0)) + qNewR3); % 1== exp(0*(kTt-qNewT0)));
    
    % Compute the new error power
    kNewErr= kNewYEst - kYData;  % Y== YEst-Err
    qNewErrPower= kNewErr'*kNewErr;
    
    % Look for improvement
    if qNewErrPower < qBestErrPower
        nnNoProgress= 0;        % Reset no progress count

        %Check redux
        qRedux= 1-(qNewErrPower/qBestErrPower);
        if qRedux < MINPOWREDUX
            bolReduxFlag2= bolReduxFlag1; bolReduxFlag1= bolReduxFlag0;
            bolReduxFlag0= TRUE;        % Set flag
        else
            bolReduxFlag0= FALSE;       % Clear flag
        end
        
        qBestErrPower= qNewErrPower;    % New best values
        kBestYEst= kNewYEst;
        kBestErr= kNewErr;
        
        qBestGain= qNewGain;
        qBestT0= qNewT0;
        qBestR1= qNewR1;
        qBestP1= qNewP1;
        qBestR3= qNewR3;
        
        nnSol= nnSearch;      % New metadata
        qSolTime= datetime;
        
        %Decrease SearchSigma
        qSearchSigma= qSearchSigma*SIGMASTEP;
        
        % progress plot
        figure(1)
        plot(kTt, kYData, kTt, kBestYEst);
        xlabel(sprintf('nn= %d, Log err power= %8.4f, qRedux= %6.5f ',nnSearch, log10(qBestErrPower), qRedux))
        title(DATAFILE);
        pause(0.1)
        
    end
    
    %Stop search if no error power reduction
    if (bolReduxFlag0 && bolReduxFlag1 &&bolReduxFlag2 ); EXREASON= NOPOWREDUX; bolSearch= FALSE; break; end
    
    %Stop search if no progress
    if nnNoProgress > MAXNOPROGRESS; EXREASON= NOPROGRESS; bolSearch= FALSE; break; end
    
    % Stop search if to many iterations
    if nnSearch > NNMAX; EXREASON= TOOMANYITERS; bolSearch= FALSE; break; end
    
    % Stop search if SearchSigma is too small
    if qSearchSigma < MINSEARCHSIGMA; EXREASON= TOOSMALLSIGMA; bolSearch= FALSE; break; end
end


figure(2)
plot(kTt, kYData, kTt, kBestYEst);
xlabel(sprintf('Sol: nn= %d, Log err power= %8.4f, qRedux= %6.5f',nnSol, log10(qBestErrPower), qRedux))
title(DATAFILE);
pause(0.1)

%------------------------------------------

fprintf('qBestGain= %7.4f\n',qBestGain);
fprintf('qBestT0= %7.4f\n',qBestT0);
fprintf('qBestR1= %7.4f\n',qBestR1);
fprintf('qBestP1= %7.4f\n',qBestP1);
fprintf('qBestR3= %7.4f\n',qBestR3);


disp(' ')
qEndTime= datetime;
qRunElapsedTime= between(qStartTime, qEndTime);
qSolElasedTime= between(qStartTime, qSolTime);
disp('Solution found after: ');
disp(qSolElasedTime);
disp(' ')
disp('Run duration: ');
disp(qRunElapsedTime);


% End: play ten beeps 
% [y, fs] = audioread('C:\Windows\Media\Windows Ding.wav');
% for jj= 1:10; sound(10*y, fs); pause(0.5); end


